import random
import string

import django


def random_string(size=6, chars=string.ascii_uppercase + string.digits):
    return ''.join(random.choice(chars) for x in range(size))


def configure_django():
    from django.conf import settings

    try:
        settings.configure(
            DEBUG_PROPAGATE_EXCEPTIONS=True,
            DATABASES={
                'default': {
                    'ENGINE': 'django.db.backends.sqlite3',
                    'NAME': ':memory:'
                }
            },
            SITE_ID=1,
            SECRET_KEY='not very secret in tests',
            USE_I18N=True,
            USE_L10N=True,
            STATIC_URL='/static/',
            ROOT_URLCONF='tests.urls',
            TEMPLATES=[
                {
                    'BACKEND': 'django.template.backends.django.DjangoTemplates',
                    'APP_DIRS': True,
                    'OPTIONS': {
                        "debug": True,  # We want template errors to raise
                    }
                },
            ],
            MIDDLEWARE=(
                'django.middleware.common.CommonMiddleware',
                'django.contrib.sessions.middleware.SessionMiddleware',
                'django.contrib.auth.middleware.AuthenticationMiddleware',
                'django.contrib.messages.middleware.MessageMiddleware',
            ),
            INSTALLED_APPS=(
                'django.contrib.admin',
                'django.contrib.auth',
                'django.contrib.contenttypes',
                'django.contrib.sessions',
                'django.contrib.sites',
                'django.contrib.staticfiles',
                'rest_framework',
                'rest_framework.authtoken',
                'asterism',
            ),
            PASSWORD_HASHERS=(
                'django.contrib.auth.hashers.MD5PasswordHasher',
            ),
        )

        django.setup()
    except RuntimeError:
        pass



from django.db import models


class BasePackage(models.Model):
    """
    Base Package model which can be subclassed to add additional fields.

    PROCESS_STATUS_CHOICES is expected to be overridden by child models.
    """
    bag_identifier = models.CharField(max_length=255, unique=True)
    bag_path = models.CharField(max_length=255, null=True, blank=True)
    TYPE_CHOICES = (
        ('aip', 'Archival Information Package'),
        ('dip', 'Dissemination Information Package'),
    )
    type = models.CharField(
        max_length=50,
        choices=TYPE_CHOICES,
        null=True,
        blank=True)
    ORIGIN_CHOICES = (
        ('aurora', 'Aurora'),
        ('legacy_digital', 'Legacy Digital Processing'),
        ('digitization', 'Digitization')
    )
    origin = models.CharField(
        max_length=20,
        choices=ORIGIN_CHOICES,
        default='aurora')
    PROCESS_STATUS_CHOICES = (None)
    process_status = models.IntegerField(choices=PROCESS_STATUS_CHOICES)
    data = models.JSONField(null=True, blank=True)
    created = models.DateTimeField(auto_now_add=True)
    last_modified = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True



from unittest import TestCase

from asterism import views
from rest_framework.test import APIRequestFactory

from .helpers import configure_django

# this has to be called before we try to import anything from DRF
configure_django()


OBJECTS = ['123', '456']
MESSAGE = "this is a wake-up call"


class Routine:
    def run(self):
        return (MESSAGE, OBJECTS)


class BadRoutine:
    def run(self):
        raise Exception(MESSAGE, OBJECTS)


class ServiceView(views.BaseServiceView):
    def get_service_response(self, request):
        return (MESSAGE, OBJECTS)


class BadServiceView(views.BaseServiceView):
    def get_service_response(self, request):
        raise Exception(MESSAGE, OBJECTS)


class RoutineView(views.RoutineView):
    routine = Routine


class BadRoutineView(views.RoutineView):
    routine = BadRoutine


class TestViews(TestCase):
    def test_prepare_response(self):
        INPUTS = (
            ("This is a string", 0),
            (("This is a tuple", "object1"), 1),
            (Exception("This is an Exception", ["object2", "object3"]), 2)
        )
        for i in INPUTS:
            response = views.prepare_response(i[0])
            self.assertIsInstance(response, dict)
            self.assertIsInstance(response['detail'], str)
            self.assertIsInstance(response['objects'], list)
            self.assertIsInstance(response['count'], int)
            self.assertEqual(response['count'], i[1])


class TestViewClasses(TestCase):
    def setUp(self):
        self.factory = APIRequestFactory()

    def test_views(self):
        for view, expected_code in [
                (ServiceView, 200),
                (BadServiceView, 500),
                (RoutineView, 200),
                (BadRoutineView, 500)]:
            request = self.factory.post(
                '/', '{}', content_type='application/json')
            response = view.as_view()(request)
            self.assertEqual(response.status_code, expected_code)
            self.assertEqual(response.data.get("objects"), OBJECTS)
            self.assertEqual(response.data.get("detail"), MESSAGE)

    def test_ping_view(self):
        request = self.factory.get('/', None, content_type='application/json')
        response = views.PingView.as_view()(request)
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.data, {'pong': True})



import datetime
import os
from shutil import copytree, rmtree
from unittest import TestCase

from asterism import file_helpers


class TestFileHelpers(TestCase):
    def setUp(self):
        self.fixtures_dir = os.path.join(
            os.getcwd(), 'asterism', 'fixtures')
        self.file_path = os.path.join(
            self.fixtures_dir, 'file_helpers', 'file.txt')
        self.dir_path = os.path.join(
            self.fixtures_dir, 'file_helpers', 'directory')
        self.tmp_dir = os.path.join(self.fixtures_dir, 'tmp')

    def test_file_attributes(self):
        for fp in os.listdir(self.fixtures_dir):
            path = os.path.join(self.fixtures_dir, fp)
            owner = file_helpers.get_owner(path)
            self.assertIsNot(False, owner)
            mtime = file_helpers.get_modified_time(path)
            self.assertTrue(isinstance(mtime, datetime.datetime))
            exists = file_helpers.is_dir_or_file(path)
            self.assertTrue(exists)

        size = file_helpers.get_dir_size(self.dir_path)
        self.assertIsNot(False, size)
        self.assertTrue(size > 0)

    def test_file_actions(self):
        target_path = os.path.join(self.tmp_dir, "directory")
        copied = file_helpers.copy_file_or_dir(self.dir_path, target_path)
        self.assertTrue(copied)
        self.assertTrue(os.path.isdir(target_path))
        self.assertTrue(os.path.isdir(self.dir_path))

        target_path = os.path.join(
            self.tmp_dir, os.path.join(self.tmp_dir, os.path.basename(self.file_path)))
        copied = file_helpers.copy_file_or_dir(self.file_path, target_path)
        self.assertTrue(copied)
        self.assertTrue(os.path.isfile(target_path))
        self.assertTrue(os.path.isfile(self.file_path))

        for path in [os.path.basename(
                self.file_path), os.path.basename(self.dir_path)]:
            moved = file_helpers.move_file_or_dir(
                os.path.join(self.tmp_dir, path), os.path.join(self.tmp_dir, "moved", path))
            self.assertTrue(moved)

        for path in os.listdir(os.path.join(self.tmp_dir, "moved")):
            removed = file_helpers.remove_file_or_dir(
                os.path.join(self.tmp_dir, "moved", path))
            self.assertTrue(removed)

    def test_make_tarfile(self):
        src_path = os.path.join(self.fixtures_dir, "file_helpers", "archive")
        for remove_src in [False, True]:
            for fp, compressed in [
                    ("archive.tar", False), ("archive.tar.gz", True)]:
                if not os.path.isdir(src_path):
                    copytree(self.dir_path, src_path)
                tarfile = file_helpers.make_tarfile(
                    src_path,
                    os.path.join(self.tmp_dir, fp),
                    compressed=compressed,
                    remove_src=remove_src)
                self.assertTrue(tarfile)

    def test_extract_serialized(self):
        for f in [
                "directory", "directory.zip",
                "directory.tar", "directory.tar.gz"]:
            extracted = file_helpers.anon_extract_all(
                os.path.join(self.fixtures_dir, "file_helpers", f), self.tmp_dir)
            self.assertTrue(extracted)
            self.assertTrue(os.path.isdir(extracted))

    def tearDown(self):
        if os.path.isdir(self.tmp_dir):
            rmtree(self.tmp_dir)



import os
from random import sample
from shutil import copytree, rmtree
from unittest import TestCase

import bagit
from asterism import bagit_helpers


class TestBagitHelpers(TestCase):
    def setUp(self):
        self.bag_path = os.path.join(
            os.getcwd(), 'asterism', 'fixtures', 'test_bag')
        if os.path.isdir(self.bag_path):
            rmtree(self.bag_path)
        copytree(
            os.path.join(os.getcwd(), 'asterism', 'fixtures', 'bag'),
            self.bag_path)
        bagit.make_bag(self.bag_path)

    def remove_random_file(self, dir):
        for f in sample(os.listdir(dir), 1):
            os.remove(os.path.join(dir, f))

    def test_validate(self):
        valid = bagit_helpers.validate(self.bag_path)
        self.assertEqual(valid, True)
        self.remove_random_file(os.path.join(self.bag_path, 'data'))
        with self.assertRaises(bagit.BagValidationError):
            bagit_helpers.validate(self.bag_path)

    def test_update_bag_info(self):
        key = "foo"
        value = "bar"
        bagit_helpers.update_bag_info(self.bag_path, {key: value})
        bag = bagit.Bag(self.bag_path)
        self.assertEqual(bag.info[key], value)
        with self.assertRaises(AssertionError):
            bagit_helpers.update_bag_info(self.bag_path, [key, value])

    def test_update_manifests(self):
        self.remove_random_file(os.path.join(self.bag_path, 'data'))
        bagit_helpers.update_manifests(self.bag_path)
        bag = bagit.Bag(self.bag_path)
        self.assertTrue(bag.validate())

    def test_get_bag_info_fields(self):
        fields = bagit_helpers.get_bag_info_fields(self.bag_path)
        self.assertTrue(
            isinstance(
                fields,
                dict),
            "Incorrect bag-info data: {}".format(fields))
        for key in ["Bag_Software_Agent", "Bagging_Date", "Payload_Oxum"]:
            self.assertTrue(key in fields.keys())

    def tearDown(self):
        rmtree(self.bag_path)



from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(name='asterism',
      version='0.8.0',
      description='Helpers for Project Electron infrastructure',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='http://github.com/RockefellerArchiveCenter/asterism',
      author='Rockefeller Archive Center',
      author_email='archive@rockarch.org',
      install_requires=[
          'bagit',
          'django',
          'djangorestframework',
          'psycopg2-binary',
          'odin'],
      license='MIT',
      packages=find_packages(),
      test_suite='nose.collector',
      tests_require=[
          'bagit',
          'django',
          'djangorestframework',
          'psycopg2-binary',
          'nose',
          'odin'],
      zip_safe=False)



from rest_framework.response import Response
from rest_framework.views import APIView


def prepare_response(data):
    """
    Transforms a string, Exception or tuple to consistently structured output.
    """
    data = data.args if isinstance(data, Exception) else data
    if isinstance(data, str):
        detail = data
        objects = []
    else:
        detail = str(data[0])
        objects = data[1] if len(data) > 1 else []
    if objects and not isinstance(objects, list):
        objects = [objects]
    count = len(objects) if objects else 0
    return {"detail": detail, "objects": objects, "count": count}


class BaseServiceView(APIView):
    """Base view which accepts a POST request and returns either 200 OK or
    500 Internal Server Error. Requires child classes to implement a
    `get_service_response()` method."""

    def post(self, request, format=None):
        try:
            response = self.get_service_response(request)
            return Response(prepare_response(response), status=200)
        except Exception as e:
            return Response(prepare_response(e), status=500)


class RoutineView(BaseServiceView):
    """Base view for routines which expose a `run` method which executes the
    main logic."""

    def get_service_response(self, request):
        return self.routine().run()


class PingView(APIView):
    """Returns a response if the application is running."""

    def get(self, request):
        return Response({"pong": True}, status=200)
